import datetime
from _typeshed import Incomplete
from typing import Any, Literal, TypedDict, overload, type_check_only
from typing_extensions import TypeAlias

from docker._types import WaitContainerResponse
from docker.types.daemon import CancellableStream

from ..types import ContainerConfig, EndpointConfig, HostConfig, NetworkingConfig

@type_check_only
class _HasId(TypedDict):
    Id: str

@type_check_only
class _HasID(TypedDict):
    ID: str

_Container: TypeAlias = _HasId | _HasID | str

class ContainerApiMixin:
    def attach(
        self,
        container: _Container,
        stdout: bool = True,
        stderr: bool = True,
        stream: bool = False,
        logs: bool = False,
        demux: bool = False,
    ): ...
    def attach_socket(self, container: _Container, params=None, ws: bool = False): ...
    def commit(
        self,
        container: _Container,
        repository: str | None = None,
        tag: str | None = None,
        message=None,
        author=None,
        pause: bool = True,
        changes=None,
        conf=None,
    ): ...
    def containers(
        self,
        quiet: bool = False,
        all: bool = False,
        trunc: bool = False,
        latest: bool = False,
        since: str | None = None,
        before: str | None = None,
        limit: int = -1,
        size: bool = False,
        filters=None,
    ): ...
    def create_container(
        self,
        image,
        command: str | list[str] | None = None,
        hostname: str | None = None,
        user: str | int | None = None,
        detach: bool = False,
        stdin_open: bool = False,
        tty: bool = False,
        # list is invariant, enumerating all possible union combination would be too complex for:
        # list[str | int | tuple[int | str, str] | tuple[int | str, ...]]
        ports: dict[str, dict[Incomplete, Incomplete]] | list[Any] | None = None,
        environment: dict[str, str] | list[str] | None = None,
        volumes: str | list[str] | None = None,
        network_disabled: bool = False,
        name: str | None = None,
        entrypoint: str | list[str] | None = None,
        working_dir: str | None = None,
        domainname: str | None = None,
        host_config=None,
        mac_address: str | None = None,
        labels: dict[str, str] | list[str] | None = None,
        stop_signal: str | None = None,
        networking_config=None,
        healthcheck=None,
        stop_timeout: int | None = None,
        runtime: str | None = None,
        use_config_proxy: bool = True,
        platform: str | None = None,
    ): ...
    def create_container_config(self, *args, **kwargs) -> ContainerConfig: ...
    def create_container_from_config(self, config, name=None, platform=None): ...
    def create_host_config(self, *args, **kwargs) -> HostConfig: ...
    def create_networking_config(self, *args, **kwargs) -> NetworkingConfig: ...
    def create_endpoint_config(self, *args, **kwargs) -> EndpointConfig: ...
    def diff(self, container: _Container) -> list[dict[Incomplete, Incomplete]]: ...
    def export(self, container: _Container, chunk_size: int | None = 2097152): ...
    def get_archive(
        self, container: _Container, path, chunk_size: int | None = 2097152, encode_stream: bool = False
    ) -> tuple[Incomplete, Incomplete]: ...
    def inspect_container(self, container: _Container): ...
    def kill(self, container: _Container, signal: str | int | None = None) -> None: ...
    @overload
    def logs(
        self,
        container: _Container,
        stdout: bool = True,
        stderr: bool = True,
        *,
        stream: Literal[True],
        timestamps: bool = False,
        tail: Literal["all"] | int = "all",
        since: datetime.datetime | float | None = None,
        follow: bool | None = None,
        until: datetime.datetime | float | None = None,
    ) -> CancellableStream[bytes]: ...
    @overload
    def logs(
        self,
        container: _Container,
        stdout: bool,
        stderr: bool,
        stream: Literal[True],
        timestamps: bool = False,
        tail: Literal["all"] | int = "all",
        since: datetime.datetime | float | None = None,
        follow: bool | None = None,
        until: datetime.datetime | float | None = None,
    ) -> CancellableStream[bytes]: ...
    @overload
    def logs(
        self,
        container: _Container,
        stdout: bool = True,
        stderr: bool = True,
        stream: Literal[False] = False,
        timestamps: bool = False,
        tail: Literal["all"] | int = "all",
        since: datetime.datetime | float | None = None,
        follow: bool | None = None,
        until: datetime.datetime | float | None = None,
    ) -> bytes: ...
    def pause(self, container: _Container) -> None: ...
    def port(self, container: _Container, private_port: int): ...
    def put_archive(self, container: _Container, path: str, data) -> bool: ...
    def prune_containers(self, filters=None): ...
    def remove_container(self, container: _Container, v: bool = False, link: bool = False, force: bool = False) -> None: ...
    def rename(self, container: _Container, name: str) -> None: ...
    def resize(self, container: _Container, height: int, width: int) -> None: ...
    def restart(self, container: _Container, timeout: int = 10) -> None: ...
    def start(self, container: _Container) -> None: ...
    def stats(self, container: _Container, decode: bool | None = None, stream: bool = True, one_shot: bool | None = None): ...
    def stop(self, container: _Container, timeout: int | None = None) -> None: ...
    def top(self, container: _Container, ps_args: str | None = None) -> str: ...
    def unpause(self, container: _Container) -> None: ...
    def update_container(
        self,
        container: _Container,
        blkio_weight: int | None = None,
        cpu_period: int | None = None,
        cpu_quota: int | None = None,
        cpu_shares: int | None = None,
        cpuset_cpus: str | None = None,
        cpuset_mems: str | None = None,
        mem_limit: float | str | None = None,
        mem_reservation: float | str | None = None,
        memswap_limit: int | str | None = None,
        kernel_memory: int | str | None = None,
        restart_policy=None,
    ): ...
    def wait(
        self,
        container: _Container,
        timeout: int | None = None,
        condition: Literal["not-running", "next-exit", "removed"] | None = None,
    ) -> WaitContainerResponse: ...



from _typeshed import Incomplete
from typing import ClassVar, final
from typing_extensions import Self

default_pool: DescriptorPool

@final
class Arena: ...

@final
class Descriptor:
    containing_type: Incomplete
    enum_types: Incomplete
    enum_types_by_name: Incomplete
    enum_values_by_name: Incomplete
    extension_ranges: Incomplete
    extensions: Incomplete
    extensions_by_name: Incomplete
    fields: Incomplete
    fields_by_camelcase_name: Incomplete
    fields_by_name: Incomplete
    fields_by_number: Incomplete
    file: Incomplete
    full_name: Incomplete
    has_options: Incomplete
    is_extendable: Incomplete
    name: Incomplete
    nested_types: Incomplete
    nested_types_by_name: Incomplete
    oneofs: Incomplete
    oneofs_by_name: Incomplete
    def __new__(cls, *args, **kwargs) -> Self: ...  # incomplete
    def CopyToProto(self, object, /): ...
    def EnumValueName(self, *args, **kwargs): ...  # incomplete
    def GetOptions(self): ...

@final
class DescriptorPool:
    def __new__(cls, *args, **kwargs) -> Self: ...  # incomplete
    def Add(self, object, /): ...
    def AddSerializedFile(self, object, /): ...
    def FindAllExtensions(self, object, /): ...
    def FindEnumTypeByName(self, object, /): ...
    def FindExtensionByName(self, object, /): ...
    def FindExtensionByNumber(self, *args, **kwargs): ...  # incomplete
    def FindFieldByName(self, object, /): ...
    def FindFileByName(self, object, /): ...
    def FindFileContainingSymbol(self, object, /): ...
    def FindMessageTypeByName(self, object, /): ...
    def FindMethodByName(self, object, /): ...
    def FindOneofByName(self, object, /): ...
    def FindServiceByName(self, object, /): ...
    def SetFeatureSetDefaults(self, object, /): ...

@final
class EnumDescriptor:
    containing_type: Incomplete
    file: Incomplete
    full_name: Incomplete
    has_options: Incomplete
    is_closed: Incomplete
    name: Incomplete
    values: Incomplete
    values_by_name: Incomplete
    values_by_number: Incomplete
    def __new__(cls, *args, **kwargs) -> Self: ...  # incomplete
    def CopyToProto(self, object, /): ...
    def GetOptions(self): ...

@final
class EnumValueDescriptor:
    has_options: Incomplete
    index: Incomplete
    name: Incomplete
    number: Incomplete
    type: Incomplete
    def __new__(cls, *args, **kwargs) -> Self: ...  # incomplete
    def GetOptions(self): ...

@final
class ExtensionDict:
    def __contains__(self, other, /) -> bool: ...
    def __delitem__(self, other, /) -> None: ...
    def __eq__(self, other: object, /) -> bool: ...
    def __ge__(self, other: object, /) -> bool: ...
    def __getitem__(self, index, /): ...
    def __gt__(self, other: object, /) -> bool: ...
    def __iter__(self): ...
    def __le__(self, other: object, /) -> bool: ...
    def __len__(self) -> int: ...
    def __lt__(self, other: object, /) -> bool: ...
    def __ne__(self, other: object, /) -> bool: ...
    def __setitem__(self, index, object, /) -> None: ...

@final
class ExtensionIterator:
    def __iter__(self): ...
    def __next__(self): ...

@final
class FieldDescriptor:
    CPPTYPE_BOOL: ClassVar[int] = ...
    CPPTYPE_BYTES: ClassVar[int] = ...
    CPPTYPE_DOUBLE: ClassVar[int] = ...
    CPPTYPE_ENUM: ClassVar[int] = ...
    CPPTYPE_FLOAT: ClassVar[int] = ...
    CPPTYPE_INT32: ClassVar[int] = ...
    CPPTYPE_INT64: ClassVar[int] = ...
    CPPTYPE_MESSAGE: ClassVar[int] = ...
    CPPTYPE_STRING: ClassVar[int] = ...
    CPPTYPE_UINT32: ClassVar[int] = ...
    CPPTYPE_UINT64: ClassVar[int] = ...
    LABEL_OPTIONAL: ClassVar[int] = ...
    LABEL_REPEATED: ClassVar[int] = ...
    LABEL_REQUIRED: ClassVar[int] = ...
    TYPE_BOOL: ClassVar[int] = ...
    TYPE_BYTES: ClassVar[int] = ...
    TYPE_DOUBLE: ClassVar[int] = ...
    TYPE_ENUM: ClassVar[int] = ...
    TYPE_FIXED32: ClassVar[int] = ...
    TYPE_FIXED64: ClassVar[int] = ...
    TYPE_FLOAT: ClassVar[int] = ...
    TYPE_GROUP: ClassVar[int] = ...
    TYPE_INT32: ClassVar[int] = ...
    TYPE_INT64: ClassVar[int] = ...
    TYPE_MESSAGE: ClassVar[int] = ...
    TYPE_SFIXED32: ClassVar[int] = ...
    TYPE_SFIXED64: ClassVar[int] = ...
    TYPE_SINT32: ClassVar[int] = ...
    TYPE_SINT64: ClassVar[int] = ...
    TYPE_STRING: ClassVar[int] = ...
    TYPE_UINT32: ClassVar[int] = ...
    TYPE_UINT64: ClassVar[int] = ...
    camelcase_name: Incomplete
    containing_oneof: Incomplete
    containing_type: Incomplete
    cpp_type: Incomplete
    default_value: Incomplete
    enum_type: Incomplete
    extension_scope: Incomplete
    file: Incomplete
    full_name: Incomplete
    has_default_value: Incomplete
    has_options: Incomplete
    has_presence: Incomplete
    index: Incomplete
    is_extension: Incomplete
    is_packed: Incomplete
    json_name: Incomplete
    label: Incomplete
    message_type: Incomplete
    name: Incomplete
    number: Incomplete
    type: Incomplete
    def __new__(cls, *args, **kwargs) -> Self: ...  # incomplete
    def GetOptions(self): ...

@final
class FileDescriptor:
    dependencies: Incomplete
    enum_types_by_name: Incomplete
    extensions_by_name: Incomplete
    has_options: Incomplete
    message_types_by_name: Incomplete
    name: Incomplete
    package: Incomplete
    pool: Incomplete
    public_dependencies: Incomplete
    serialized_pb: Incomplete
    services_by_name: Incomplete
    def __new__(cls, *args, **kwargs) -> Self: ...  # incomplete
    def CopyToProto(self, object, /): ...
    def GetOptions(self): ...

@final
class MapIterator:
    def __iter__(self): ...
    def __next__(self): ...

@final
class Message:
    Extensions: Incomplete
    def __init__(self, *args, **kwargs) -> None: ...  # incomplete
    def ByteSize(self): ...
    def Clear(self): ...
    def ClearExtension(self, object, /): ...
    def ClearField(self, object, /): ...
    def CopyFrom(self, object, /): ...
    def DiscardUnknownFields(self): ...
    def FindInitializationErrors(self): ...
    @classmethod
    def FromString(cls, object, /): ...
    def HasExtension(self, object, /): ...
    def HasField(self, object, /): ...
    def IsInitialized(self, *args, **kwargs): ...  # incomplete
    def ListFields(self): ...
    def MergeFrom(self, object, /): ...
    def MergeFromString(self, object, /): ...
    def ParseFromString(self, object, /): ...
    def SerializePartialToString(self, *args, **kwargs): ...  # incomplete
    def SerializeToString(self, *args, **kwargs): ...  # incomplete
    def SetInParent(self): ...
    def UnknownFields(self): ...
    def WhichOneof(self, object, /): ...
    def __contains__(self, other, /) -> bool: ...
    def __deepcopy__(self, memo=None): ...
    def __delattr__(self, name, /): ...
    def __eq__(self, other: object, /) -> bool: ...
    def __ge__(self, other: object, /) -> bool: ...
    def __gt__(self, other: object, /) -> bool: ...
    def __le__(self, other: object, /) -> bool: ...
    def __lt__(self, other: object, /) -> bool: ...
    def __ne__(self, other: object, /) -> bool: ...
    def __setattr__(self, name, value, /): ...

@final
class MessageMeta(type): ...

@final
class MethodDescriptor:
    client_streaming: Incomplete
    containing_service: Incomplete
    full_name: Incomplete
    index: Incomplete
    input_type: Incomplete
    name: Incomplete
    output_type: Incomplete
    server_streaming: Incomplete
    def __new__(cls, *args, **kwargs) -> Self: ...  # incomplete
    def CopyToProto(self, object, /): ...
    def GetOptions(self): ...

@final
class OneofDescriptor:
    containing_type: Incomplete
    fields: Incomplete
    full_name: Incomplete
    has_options: Incomplete
    index: Incomplete
    name: Incomplete
    def __new__(cls, *args, **kwargs) -> Self: ...  # incomplete
    def GetOptions(self): ...

@final
class RepeatedCompositeContainer:
    def __new__(cls, *args, **kwargs) -> Self: ...  # incomplete
    def MergeFrom(self, object, /): ...
    def add(self, *args, **kwargs): ...  # incomplete
    def append(self, object, /): ...
    def extend(self, object, /): ...
    def insert(self, *args, **kwargs): ...  # incomplete
    def pop(self, *args, **kwargs): ...  # incomplete
    def remove(self, object, /): ...
    def reverse(self): ...
    def sort(self, *args, **kwargs): ...  # incomplete
    def __deepcopy__(self, memo=None): ...
    def __delitem__(self, other, /) -> None: ...
    def __eq__(self, other: object, /) -> bool: ...
    def __ge__(self, other: object, /) -> bool: ...
    def __getitem__(self, index, /): ...
    def __gt__(self, other: object, /) -> bool: ...
    def __le__(self, other: object, /) -> bool: ...
    def __len__(self) -> int: ...
    def __lt__(self, other: object, /) -> bool: ...
    def __ne__(self, other: object, /) -> bool: ...
    def __setitem__(self, index, object, /) -> None: ...

@final
class RepeatedScalarContainer:
    def __new__(cls, *args, **kwargs) -> Self: ...  # incomplete
    def MergeFrom(self, object, /): ...
    def append(self, object, /): ...
    def extend(self, object, /): ...
    def insert(self, *args, **kwargs): ...  # incomplete
    def pop(self, *args, **kwargs): ...  # incomplete
    def remove(self, object, /): ...
    def reverse(self): ...
    def sort(self, *args, **kwargs): ...  # incomplete
    def __deepcopy__(self, memo=None): ...
    def __delitem__(self, other, /) -> None: ...
    def __eq__(self, other: object, /) -> bool: ...
    def __ge__(self, other: object, /) -> bool: ...
    def __getitem__(self, index, /): ...
    def __gt__(self, other: object, /) -> bool: ...
    def __le__(self, other: object, /) -> bool: ...
    def __len__(self) -> int: ...
    def __lt__(self, other: object, /) -> bool: ...
    def __ne__(self, other: object, /) -> bool: ...
    def __reduce__(self): ...
    def __setitem__(self, index, object, /) -> None: ...

@final
class ServiceDescriptor:
    file: Incomplete
    full_name: Incomplete
    index: Incomplete
    methods: Incomplete
    methods_by_name: Incomplete
    name: Incomplete
    def __new__(cls, *args, **kwargs) -> Self: ...  # incomplete
    def CopyToProto(self, object, /): ...
    def FindMethodByName(self, object, /): ...
    def GetOptions(self): ...

@final
class UnknownFieldSet:
    def __new__(cls, *args, **kwargs) -> Self: ...  # incomplete
    def __getitem__(self, index, /): ...
    def __len__(self) -> int: ...

def SetAllowOversizeProtos(object, /): ...  # incomplete



from .lua54 import *

__all__ = [
    # from lua54 (newest lib)
    "LUA_VERSION",
    "LUA_MAXINTEGER",
    "LUA_MININTEGER",
    "LuaRuntime",
    "LuaError",
    "LuaSyntaxError",
    "LuaMemoryError",
    "as_itemgetter",
    "as_attrgetter",
    "lua_type",
    "unpacks_lua_table",
    "unpacks_lua_table_method",
]



from _typeshed import Incomplete
from collections.abc import Generator

from . import base

class Filter(base.Filter[dict[str, Incomplete]]):
    def slider(self) -> Generator[tuple[Incomplete, Incomplete, Incomplete]]: ...
    def is_optional_start(self, tagname: str, previous, next) -> bool: ...
    def is_optional_end(self, tagname: str, next) -> bool: ...



from typing import Final, Literal
from typing_extensions import TypeAlias

_BoolInt: TypeAlias = Literal[0, 1]

__version__: Final[str]

def ps2tt(psfn: str) -> tuple[str, _BoolInt, _BoolInt]: ...
def tt2ps(fn: str, b: _BoolInt, i: _BoolInt) -> str: ...
def addMapping(face: str, bold: _BoolInt, italic: _BoolInt, psname: str) -> None: ...



import socket

import urllib3
import urllib3.connection
from docker.transport.basehttpadapter import BaseHTTPAdapter
from requests import PreparedRequest
from urllib3._collections import RecentlyUsedContainer as urllib3_RecentlyUsedContainer

RecentlyUsedContainer = urllib3_RecentlyUsedContainer

class UnixHTTPConnection(urllib3.connection.HTTPConnection):
    base_url: str
    unix_socket: str
    timeout: int
    def __init__(self, base_url: str, unix_socket: str, timeout: int = 60) -> None: ...
    sock: socket.socket | None
    def connect(self) -> None: ...

class UnixHTTPConnectionPool(urllib3.connectionpool.HTTPConnectionPool):
    base_url: str
    socket_path: str
    timeout: urllib3.Timeout
    def __init__(self, base_url: str, socket_path: str, timeout: int = 60, maxsize: int = 10) -> None: ...

class UnixHTTPAdapter(BaseHTTPAdapter):
    __attrs__: list[str]
    socket_path: str
    timeout: int
    max_pool_size: int
    pools: RecentlyUsedContainer
    def __init__(self, socket_url: str, timeout: int = 60, pool_connections: int = 25, max_pool_size: int = 10) -> None: ...
    def get_connection(self, url: bytes | str, proxies=None) -> UnixHTTPConnectionPool: ...
    # proxies is unused
    def request_url(self, request: PreparedRequest, proxies) -> str: ...



from typing import Final

DAEMON_ADDRESS_KEY: Final = "AWS_XRAY_DAEMON_ADDRESS"
DEFAULT_ADDRESS: Final = "127.0.0.1:2000"

class DaemonConfig:
    def __init__(self, daemon_address: str | None = "127.0.0.1:2000") -> None: ...
    @property
    def udp_ip(self) -> str: ...
    @property
    def udp_port(self) -> int: ...
    @property
    def tcp_ip(self) -> str: ...
    @property
    def tcp_port(self) -> int: ...



from _typeshed import Incomplete
from typing import type_check_only

RequirePyRXP: int
simpleparse: int

@type_check_only
class _smartDecode:
    @staticmethod
    def __call__(s): ...

smartDecode: _smartDecode
NONAME: str
NAMEKEY: int
CONTENTSKEY: int
CDATAMARKER: str
LENCDATAMARKER: Incomplete
CDATAENDMARKER: str
replacelist: Incomplete

def unEscapeContentList(contentList): ...
def parsexmlSimple(xmltext, oneOutermostTag: int = 0, eoCB=None, entityReplacer=...): ...

parsexml = parsexmlSimple

def parseFile(filename): ...

verbose: int

def skip_prologue(text, cursor): ...
def parsexml0(xmltext, startingat: int = 0, toplevel: int = 1, entityReplacer=...): ...
def pprettyprint(parsedxml): ...
def testparse(s, dump: int = 0) -> None: ...
def test(dump: int = 0) -> None: ...



from typing import Any, overload
from typing_extensions import TypeAlias

import numpy as np
import tensorflow as tf
from tensorflow._aliases import FloatArray, IntArray

# The alias below is not fully accurate, since TensorFlow casts the inputs, they have some additional
# requirements. For example y needs to be castable into x's dtype. Moreover, x and y cannot both be booleans.
# Properly typing the bitwise functions would be overly complicated and unlikely to provide much benefits
# since most people use Tensors, it was therefore not done.
_BitwiseCompatible: TypeAlias = tf.Tensor | int | FloatArray | IntArray | np.number[Any]

@overload
def bitwise_and(x: _BitwiseCompatible, y: _BitwiseCompatible, name: str | None = None) -> tf.Tensor: ...
@overload
def bitwise_and(x: tf.RaggedTensor, y: tf.RaggedTensor, name: str | None = None) -> tf.RaggedTensor: ...
@overload
def bitwise_or(x: _BitwiseCompatible, y: _BitwiseCompatible, name: str | None = None) -> tf.Tensor: ...
@overload
def bitwise_or(x: tf.RaggedTensor, y: tf.RaggedTensor, name: str | None = None) -> tf.RaggedTensor: ...
@overload
def bitwise_xor(x: _BitwiseCompatible, y: _BitwiseCompatible, name: str | None = None) -> tf.Tensor: ...
@overload
def bitwise_xor(x: tf.RaggedTensor, y: tf.RaggedTensor, name: str | None = None) -> tf.RaggedTensor: ...
@overload
def invert(x: _BitwiseCompatible, name: str | None = None) -> tf.Tensor: ...
@overload
def invert(x: tf.RaggedTensor, name: str | None = None) -> tf.RaggedTensor: ...
@overload
def left_shift(x: _BitwiseCompatible, y: _BitwiseCompatible, name: str | None = None) -> tf.Tensor: ...
@overload
def left_shift(x: tf.RaggedTensor, y: tf.RaggedTensor, name: str | None = None) -> tf.RaggedTensor: ...
@overload
def right_shift(x: _BitwiseCompatible, y: _BitwiseCompatible, name: str | None = None) -> tf.Tensor: ...
@overload
def right_shift(x: tf.RaggedTensor, y: tf.RaggedTensor, name: str | None = None) -> tf.RaggedTensor: ...



from _typeshed import Incomplete

from pony.orm.core import Database, Entity

db: Database

class ReconciledPayments(Entity):
    id: Incomplete
    foo: Incomplete
    add_on_id: Incomplete

class ContractAddOns(Entity):
    id: Incomplete
    reconciled_payments: Incomplete

r1: ReconciledPayments
r2: ReconciledPayments
c1: ContractAddOns
old_val: Incomplete



from typing import ClassVar

from ..lexer import RegexLexer

__all__ = ["VisualPrologLexer", "VisualPrologGrammarLexer"]

class VisualPrologBaseLexer(RegexLexer):
    minorendkw: ClassVar[tuple[str, ...]]
    minorkwexp: ClassVar[tuple[str, ...]]
    dockw: ClassVar[tuple[str, ...]]

class VisualPrologLexer(VisualPrologBaseLexer):
    majorkw: ClassVar[tuple[str, ...]]
    minorkw: ClassVar[tuple[str, ...]]
    directivekw: ClassVar[tuple[str, ...]]

class VisualPrologGrammarLexer(VisualPrologBaseLexer):
    majorkw: ClassVar[tuple[str, ...]]
    directivekw: ClassVar[tuple[str, ...]]



import socket
import sys
from collections.abc import Iterable
from ssl import SSLContext, SSLSocket
from typing import Any, ClassVar, Literal, SupportsIndex

from gunicorn.glogging import Logger as GLogger

from .config import Config

class BaseSocket:
    sock: socket.socket

    def __init__(self, address: str, conf: Config, log: GLogger, fd: SupportsIndex | None = None) -> None: ...
    def __getattr__(self, name: str) -> Any: ...
    def set_options(self, sock: socket.socket, bound: bool = False) -> socket.socket: ...
    def bind(self, sock: socket.socket) -> None: ...
    def close(self) -> None: ...

class TCPSocket(BaseSocket):
    FAMILY: ClassVar[Literal[socket.AddressFamily.AF_INET, socket.AddressFamily.AF_INET6]]

    def set_options(self, sock: socket.socket, bound: bool = False) -> socket.socket: ...

class TCP6Socket(TCPSocket):
    FAMILY: ClassVar[Literal[socket.AddressFamily.AF_INET6]]

class UnixSocket(BaseSocket):
    if sys.platform != "win32":
        FAMILY: ClassVar[Literal[socket.AddressFamily.AF_UNIX]]
    else:
        FAMILY: ClassVar[Literal[0]]  # Stub for windows

    def __init__(self, addr: str, conf: Config, log: GLogger, fd: SupportsIndex | None = None) -> None: ...
    def bind(self, sock: socket.socket) -> None: ...

def create_sockets(conf: Config, log: GLogger, fds: Iterable[SupportsIndex] | None = None) -> list[BaseSocket]: ...
def close_sockets(listeners: Iterable[socket.socket], unlink: bool = True) -> None: ...
def ssl_context(conf: Config) -> SSLContext: ...
def ssl_wrap_socket(sock: socket.socket, conf: Config) -> SSLSocket: ...



import logging
from typing import Any

log: logging.Logger

class ImageApiMixin:
    def get_image(self, image: str, chunk_size: int | None = 2097152): ...
    def history(self, image): ...
    def images(self, name: str | None = None, quiet: bool = False, all: bool = False, filters=None): ...
    def import_image(
        self,
        src=None,
        repository: str | None = None,
        tag: str | None = None,
        image: str | None = None,
        changes=None,
        stream_src: bool = False,
    ): ...
    def import_image_from_data(self, data, repository: str | None = None, tag: str | None = None, changes=None): ...
    def import_image_from_file(self, filename: str, repository: str | None = None, tag: str | None = None, changes=None): ...
    def import_image_from_stream(self, stream, repository: str | None = None, tag: str | None = None, changes=None): ...
    def import_image_from_url(self, url, repository: str | None = None, tag: str | None = None, changes=None): ...
    def import_image_from_image(self, image, repository: str | None = None, tag: str | None = None, changes=None): ...
    def inspect_image(self, image): ...
    def inspect_distribution(self, image, auth_config=None): ...
    def load_image(self, data, quiet=None): ...
    def prune_images(self, filters=None): ...
    def pull(
        self,
        repository: str,
        tag: str | None = None,
        stream: bool = False,
        auth_config: dict[str, Any] | None = None,
        decode: bool = False,
        platform: str | None = None,
        all_tags: bool = False,
    ): ...
    def push(self, repository: str, tag: str | None = None, stream: bool = False, auth_config=None, decode: bool = False): ...
    def remove_image(self, image: str, force: bool = False, noprune: bool = False): ...
    def search(self, term: str, limit: int | None = None): ...
    def tag(self, image, repository, tag: str | None = None, force: bool = False): ...

def is_file(src: str) -> bool: ...
